@@ -29,10 +29,12 @@
 #include <sys/types.h>
 #include <fcntl.h> 
 #include <load.h>
+#include <notify.h>
+#include <memory.h>
+#include <perl.h>
 #include <errno.h>
 #include <util.h>
 #include <http.h>
-#include <joedog/joedog.h>
 #include <joedog/boolean.h>
 
 struct ContentType {
@@ -61,6 +63,7 @@ static const struct ContentType tmap[] = {
   {"cpt",     FALSE, "application/mac-compactpro"},
   {"csh",     FALSE, "application/x-csh"},
   {"css",     TRUE,  "text/css"},
+  {"csv",     TRUE,  "text/csv"},
   {"dcr",     FALSE, "application/x-director"},
   {"dir",     FALSE, "application/x-director"},
   {"dms",     FALSE, "application/octet-stream"},
@@ -87,6 +90,7 @@ static const struct ContentType tmap[] = {
   {"htm",     TRUE,  "text/html"},
   {"html",    TRUE,  "text/html"},
   {"ice",     FALSE, "x-conference/x-cooltalk"},
+  {"ico",     FALSE, "image/x-icon"},
   {"ief",     FALSE, "image/ief"},
   {"iges",    FALSE, "model/iges"},
   {"igs",     FALSE, "model/iges"},
@@ -104,6 +108,7 @@ static const struct ContentType tmap[] = {
   {"lzh",     FALSE, "application/octet-stream"},
   {"m",       TRUE,  "text/plain"},
   {"man",     FALSE, "application/x-troff-man"},
+  {"md",      TRUE,  "text/x-markdown"},
   {"me",      FALSE, "application/x-troff-me"},
   {"mesh",    FALSE, "model/mesh"},
   {"mid",     FALSE, "audio/midi"},
@@ -170,6 +175,7 @@ static const struct ContentType tmap[] = {
   {"stp",     FALSE, "application/STEP"},
   {"sv4cpio", FALSE, "application/x-sv4cpio"},
   {"sv4crc",  FALSE, "application/x-sv4crc"},
+  {"svg",     TRUE,  "image/svg+xml"},
   {"swf",     FALSE, "application/x-shockwave-flash"},
   {"t",       FALSE, "application/x-troff"},
   {"tar",     FALSE, "application/x-tar"},
@@ -203,6 +209,7 @@ static const struct ContentType tmap[] = {
   {"xpm",     FALSE, "image/x-xpixmap"},
   {"xwd",     FALSE, "image/x-xwindowdump"},
   {"xyz",     FALSE, "chemical/x-pdb"},
+  {"yml",     TRUE,  "application/x-yaml"},
   {"zip",     FALSE, "application/zip"}
 };
 
@@ -266,6 +273,7 @@ load_file(URL U, char *file)
 
   if ((fread(buf, 1, len, fp )) == len) {
     if (is_ascii(filename)) {
+      buf[len] = '\0';
       trim(buf);
       len = strlen(buf);
     }
