@@ -1,7 +1,7 @@
 /**
  * Siege, http regression tester / benchmark utility
  *
- * Copyright (C) 2000-2014 by  
+ * Copyright (C) 2000-2015 by  
  * Jeffrey Fulmer - <jeff@joedog.org>, et al. 
  * This file is distributed as part of Siege
  *
@@ -35,17 +35,19 @@
 #include <array.h>
 #include <handler.h>
 #include <timer.h>
-#include <client.h>
+#include <browser.h>
 #include <util.h>
 #include <log.h>
 #include <init.h>
 #include <cfg.h>
 #include <url.h>
 #include <ssl.h>
-#include <cookie.h>
+#include <cookies.h>
 #include <crew.h>
 #include <data.h>
 #include <version.h>
+#include <memory.h>
+#include <notify.h>
 #include <sys/resource.h>
 #ifdef __CYGWIN__
 # include <getopt.h>
@@ -136,18 +138,18 @@ display_help()
   puts("  -g, --get                 GET, pull down HTTP headers and display the");
   puts("                            transaction. Great for application debugging.");
   puts("  -c, --concurrent=NUM      CONCURRENT users, default is 10");
-  puts("  -i, --internet            INTERNET user simulation, hits URLs randomly." );
-  puts("  -b, --benchmark           BENCHMARK: no delays between requests." );
-  puts("  -t, --time=NUMm           TIMED testing where \"m\" is modifier S, M, or H" );
-  puts("                            ex: --time=1H, one hour test." );
   puts("  -r, --reps=NUM            REPS, number of times to run the test." );
+  puts("  -t, --time=NUMm           TIMED testing where \"m\" is modifier S, M, or H");
+  puts("                            ex: --time=1H, one hour test." );
+  puts("  -d, --delay=NUM           Time DELAY, random delay before each requst");
+  puts("  -b, --benchmark           BENCHMARK: no delays between requests." );
+  puts("  -i, --internet            INTERNET user simulation, hits URLs randomly.");
   puts("  -f, --file=FILE           FILE, select a specific URLS FILE." );
   printf("  -R, --rc=FILE             RC, specify an %src file\n",program_name);
   puts("  -l, --log[=FILE]          LOG to FILE. If FILE is not specified, the");
   printf("                            default is used: PREFIX/var/%s.log\n", program_name);
   puts("  -m, --mark=\"text\"         MARK, mark the log file with a string." );
-  puts("  -d, --delay=NUM           Time DELAY, random delay before each requst");
-  puts("                            between 1 and NUM. (NOT COUNTED IN STATS)");
+  puts("                            between .001 and NUM. (NOT COUNTED IN STATS)");
   puts("  -H, --header=\"text\"       Add a header to request (can be many)" ); 
   puts("  -A, --user-agent=\"text\"   Sets User-Agent in request" ); 
   puts("  -T, --content-type=\"text\" Sets Content-Type in request" ); 
@@ -216,7 +218,7 @@ parse_cmdline(int argc, char *argv[])
         break;
       case 'd':
 	/* XXX range checking? use strtol? */
-        my.delay   = atoi(optarg);
+        my.delay   = atof(optarg);
 	if(my.delay < 0){
 	  my.delay = 0; 
 	}
@@ -290,21 +292,9 @@ parse_cmdline(int argc, char *argv[])
   return;
 } /* end of parse_cmdline */
 
-int 
-main(int argc, char *argv[])
+private void
+__signal_setup()
 {
-  int            x; 
-  int            j = 0;
-  int            result;
-  DATA           D    = new_data();
-  ARRAY          urls = new_array();
-  CREW           crew;  
-  LINES          *lines;   
-  CLIENT         *client; 
-  pthread_t      cease; 
-  pthread_t      timer;  
-  pthread_attr_t scope_attr; 
-  void *statusp;
   sigset_t sigs;
 
   sigemptyset(&sigs);
@@ -312,62 +302,97 @@ main(int argc, char *argv[])
   sigaddset(&sigs, SIGINT);
   sigaddset(&sigs, SIGALRM);
   sigaddset(&sigs, SIGTERM);
+  sigaddset(&sigs, SIGPIPE);
   sigprocmask(SIG_BLOCK, &sigs, NULL);
+}
 
-  lines = xcalloc(1, sizeof *lines);
-  lines->index   = 0;
-  lines->line    = NULL;
-
-  memset(&my, 0, sizeof(struct CONFIG));
+private void
+__config_setup(int argc, char *argv[])
+{
+  
+  memset(&my, '\0', sizeof(struct CONFIG));
 
   parse_rc_cmdline(argc, argv); 
-  if (init_config() < 0) {      /* defined in init.h   */
-    exit( EXIT_FAILURE );       /* polly was a girl... */
+  if (init_config() < 0) { 
+    exit(EXIT_FAILURE); 
   } 
-  parse_cmdline(argc, argv);    /* defined above       */
-  ds_module_check();            /* check config integ  */
-
-  /**
-   * XXX: we should consider moving the following
-   * if-checks into the ds_module_check
-   */
-
+  parse_cmdline(argc, argv);
+  ds_module_check(); 
+  
   if (my.config) {
     show_config(TRUE);    
   }
 
+  /** 
+   * Let's tap the brakes and make sure the user knows what they're doing...
+   */ 
+  if (my.cusers > my.limit) {
+    printf("\n");
+    printf("================================================================\n");
+    printf("WARNING: The number of users is capped at %d.%sTo increase this\n", my.limit, (my.limit>999)?" ":"  ");
+    printf("         limit, search your .siegerc file for 'limit' and change\n");
+    printf("         its value. Make sure you read the instructions there...\n");
+    printf("================================================================\n");
+    sleep(10);
+    my.cusers = my.limit;
+  }
+}
+
+private LINES *
+__urls_setup() 
+{
+  LINES * lines;
+
+  lines          = xcalloc(1, sizeof(LINES));
+  lines->index   = 0;
+  lines->line    = NULL;
+
   if (my.url != NULL) {
     my.length = 1; 
   } else { 
     my.length = read_cfg_file(lines, my.file); 
   }
 
-  //if (my.reps < 0) {
-  //  my.reps = my.length;
-  //}
-
   if (my.length == 0) { 
     display_help();
   }
 
-  /* cookie is an EXTERN, defined in setup */ 
-  cookie = xcalloc(sizeof(COOKIE), 1); 
-  cookie->first = NULL;
-  if ((result = pthread_mutex_init( &(cookie->mutex), NULL)) !=0) {
-    NOTIFY(FATAL, "pthread_mutex_init" );
-  } 
+  return lines;
+}
 
-  /* memory allocation for threads and clients */
-  client = xcalloc(my.cusers, sizeof(CLIENT));
-  if ((crew = new_crew(my.cusers, my.cusers, FALSE)) == NULL) {
-    NOTIFY(FATAL, "unable to allocate memory for %d simulated browser", my.cusers);  
-  }
+int 
+main(int argc, char *argv[])
+{
+  int       i, j;
+  int       result   = 0;
+  void  *   status   = NULL;
+  LINES *   lines    = NULL;
+  CREW      crew     = NULL;
+  DATA      data     = NULL;
+  HASH      cookies  = NULL;
+  ARRAY     urls     = new_array();
+  ARRAY     browsers = new_array();
+  pthread_t cease; 
+  pthread_t timer;  
+  pthread_attr_t scope_attr;
+ 
+  __signal_setup();
+  __config_setup(argc, argv);
+  lines = __urls_setup();
 
-  /** 
-   * determine the source of the url(s),
-   * command line or file, and add them
-   * to the urls struct.
+  pthread_attr_init(&scope_attr);
+  pthread_attr_setscope(&scope_attr, PTHREAD_SCOPE_SYSTEM);
+#if defined(_AIX)
+  /**
+   * AIX, for whatever reason, defies the pthreads standard and  
+   * creates threads detached by default. (see pthread.h on AIX) 
    */
+  pthread_attr_setdetachstate(&scope_attr, PTHREAD_CREATE_JOINABLE);
+#endif
+
+#ifdef HAVE_SSL
+  SSL_thread_setup();
+#endif
 
   if (my.url != NULL) {
     URL tmp = new_url(my.url);
@@ -377,59 +402,48 @@ main(int argc, char *argv[])
     }
     array_npush(urls, tmp, URLSIZE); // from cmd line
   } else { 
-    for (x = 0; x < my.length; x++) {
-      URL tmp = new_url(lines->line[x]);
-      url_set_ID(tmp, x);
+    for (i = 0; i < my.length; i++) {
+      URL tmp = new_url(lines->line[i]);
+      url_set_ID(tmp, i);
       array_npush(urls, tmp, URLSIZE);
     }
   } 
 
-  /**
-   * display information about the siege
-   * to the user and prepare for verbose 
-   * output if necessary.
-   */
-  if (!my.get && !my.quiet) {
-    fprintf(stderr, "** "); 
-    display_version(FALSE);
-    fprintf(stderr, "** Preparing %d concurrent users for battle.\n", my.cusers);
-    fprintf(stderr, "The server is now under siege...");
-    if (my.verbose) { fprintf(stderr, "\n"); }
+  cookies = load_cookies(my.cookies);
+
+  for (i = 0; i < my.cusers; i++) {
+    char    tmp[4096];
+    BROWSER B = new_browser(i);
+    memset(tmp, '\0', sizeof(tmp));
+    snprintf(tmp, 4096, "%d", i);
+    if (cookies != NULL) {
+      if (hash_get(cookies, tmp) != NULL) {
+        browser_set_cookies(B, (HASH)hash_get(cookies, tmp));
+      } 
+    }
+    if (my.reps > 0 ) {
+      browser_set_urls(B, urls);
+    } else {
+      /**
+       * Scenario: -r once/--reps=once 
+       */
+      int   len = (array_length(urls)/my.cusers); 
+      ARRAY tmp = new_array();
+      for (j = 0; j < ((i+1) * len) && j < (int)array_length(urls); j++) {
+        URL u = array_get(urls, j);
+        if (u != NULL && url_get_hostname(u) != NULL && strlen(url_get_hostname(u)) > 1) {
+          array_npush(tmp, array_get(urls, j), URLSIZE);    
+        }
+      } 
+      browser_set_urls(B, urls);
+    }
+    array_npush(browsers, B, BROWSERSIZE);
   }
 
-  /**
-   * record start time before spawning threads
-   * as the threads begin hitting the server as
-   * soon as they are created.
-   */
-  data_set_start(D);
+  if ((crew = new_crew(my.cusers, my.cusers, FALSE)) == NULL) {
+    NOTIFY(FATAL, "unable to allocate memory for %d simulated browser", my.cusers);  
+  } 
 
-  /**
-   * for each concurrent user, spawn a thread and
-   * loop until condition or pthread_cancel from the
-   * handler thread.
-   */
-  pthread_attr_init(&scope_attr);
-  pthread_attr_setscope(&scope_attr, PTHREAD_SCOPE_SYSTEM);
-#if defined(_AIX)
-  /* AIX, for whatever reason, defies the pthreads standard and  *
-   * creates threads detached by default. (see pthread.h on AIX) */
-  pthread_attr_setdetachstate(&scope_attr, PTHREAD_CREATE_JOINABLE);
-#endif
-
-  /** 
-   * invoke OpenSSL's thread safety
-   */
-#ifdef HAVE_SSL
-  SSL_thread_setup();
-#endif
-
-  /**
-   * create the signal handler and timer;  the
-   * signal handler thread (cease) responds to
-   * ctrl-C (sigterm) and the timer thread sends
-   * sigterm to cease on time out.
-   */
   if ((result = pthread_create(&cease, NULL, (void*)sig_handler, (void*)crew)) < 0) {
     NOTIFY(FATAL, "failed to create handler: %d\n", result);
   }
@@ -440,43 +454,22 @@ main(int argc, char *argv[])
   }
 
   /**
-   * loop until my.cusers and create a corresponding thread...
-   */  
-  for (x = 0; x < my.cusers && crew_get_shutdown(crew) != TRUE; x++) {
-    client[x].id              = x; 
-    client[x].bytes           = 0;
-    client[x].time            = 0.0;
-    client[x].hits            = 0;
-    client[x].code            = 0;
-    client[x].ok200           = 0;   
-    client[x].fail            = 0; 
-    if (my.reps > 0 ) {
-      /** 
-       * Traditional -r/--reps where each user
-       * loops through every URL in the file. 
-       */
-      client[x].urls = urls;
-    } else {
-      /**
-       * -r once/--reps=once where each URL
-       * in the file is hit only once...
-       */
-      int   len = (array_length(urls)/my.cusers); 
-      ARRAY tmp = new_array();
-      for ( ; j < ((x+1) * len) && j < (int)array_length(urls); j++) {
-        URL u = array_get(urls, j);
-        if (u != NULL && url_get_hostname(u) != NULL && strlen(url_get_hostname(u)) > 1) {
-          array_npush(tmp, array_get(urls, j), URLSIZE);    
-        }
-      } 
-      client[x].urls = tmp;
-    }
-    client[x].auth.www        = 0;
-    client[x].auth.proxy      = 0;
-    client[x].auth.type.www   = BASIC;
-    client[x].auth.type.proxy = BASIC;
-    client[x].rand_r_SEED     = urandom();
-    result = crew_add(crew, (void*)start_routine, &(client[x]));
+   * Display information about the siege to the user
+   * and prepare for verbose output if necessary.
+   */
+  if (!my.get && !my.quiet) {
+    fprintf(stderr, "** "); 
+    display_version(FALSE);
+    fprintf(stderr, "** Preparing %d concurrent users for battle.\n", my.cusers);
+    fprintf(stderr, "The server is now under siege...");
+    if (my.verbose) { fprintf(stderr, "\n"); }
+  } 
+
+  data = new_data();
+  data_set_start(data);
+  for (i = 0; i < my.cusers && crew_get_shutdown(crew) != TRUE; i++) {
+    BROWSER B = (BROWSER)array_get(browsers, i);
+    result = crew_add(crew, (void*)start, B);
     if (result == FALSE) { 
       my.verbose = FALSE;
       fprintf(stderr, "Unable to spawn additional threads; you may need to\n");
@@ -484,104 +477,74 @@ main(int argc, char *argv[])
       fprintf(stderr, "to exceed %d users.\n", my.cusers);
       NOTIFY(FATAL, "system resources exhausted"); 
     }
-  } /* end of for pthread_create */
-
-  crew_join(crew, TRUE, &statusp);
+  } 
+  crew_join(crew, TRUE, &status);
+  data_set_stop(data); 
 
 #ifdef HAVE_SSL
   SSL_thread_cleanup();
 #endif
 
-  /**
-   * collect all the data from all the threads that
-   * were spawned by the run.
-   */
-  for (x = 0; x < ((crew_get_total(crew) > my.cusers || 
-                    crew_get_total(crew)==0 ) ? my.cusers : crew_get_total(crew)); x++) {
-    data_increment_count(D, client[x].hits);
-    data_increment_bytes(D, client[x].bytes);
-    data_increment_total(D, client[x].time);
-    data_increment_code (D, client[x].code);
-    data_increment_ok200(D, client[x].ok200);
-    data_increment_fail (D, client[x].fail);
-    data_set_highest    (D, client[x].himark);
-    data_set_lowest     (D, client[x].lomark);
-    client[x].rand_r_SEED = urandom();
-  } /* end of stats accumulation */
-  
-  /**
-   * record stop time
-   */
-  data_set_stop(D);
+  for (i = 0; i < ((crew_get_total(crew) > my.cusers || 
+                    crew_get_total(crew) == 0) ? my.cusers : crew_get_total(crew)); i++) {
+    BROWSER B = (BROWSER)array_get(browsers, i);
+    data_increment_count(data, browser_get_hits(B));
+    data_increment_bytes(data, browser_get_bytes(B));
+    data_increment_total(data, browser_get_time(B));
+    data_increment_code (data, browser_get_code(B));
+    data_increment_okay (data, browser_get_okay(B));
+    data_increment_fail (data, browser_get_fail(B));
+    data_set_highest    (data, browser_get_himark(B));
+    data_set_lowest     (data, browser_get_lomark(B));
+  } crew_destroy(crew);
 
-  /**
-   * cleanup crew
-   */ 
-  crew_destroy(crew);
+  pthread_usleep_np(10000);
 
-  for (x = 0; x < my.cusers; x++) {
-    // XXX: TODO
-    //digest_challenge_destroy(client[x].auth.wwwchlg);
-    //digest_credential_destroy(client[x].auth.wwwcred);
-    //digest_challenge_destroy(client[x].auth.proxychlg);
-    //digest_credential_destroy(client[x].auth.proxycred);
+  if (!my.quiet) {
+    if (my.failures > 0 && my.failed >= my.failures) {
+      fprintf(stderr, "%s aborted due to excessive socket failure; you\n", program_name);
+      fprintf(stderr, "can change the failure threshold in $HOME/.%src\n", program_name);
+    }
+    fprintf(stderr, "\nTransactions:\t\t%12u hits\n",        data_get_count(data));
+    fprintf(stderr, "Availability:\t\t%12.2f %%\n",          data_get_count(data)==0 ? 0 :
+                                                             (double)data_get_count(data) /
+                                                             (data_get_count(data)+my.failed)*100
+    );
+    fprintf(stderr, "Elapsed time:\t\t%12.2f secs\n",        data_get_elapsed(data));
+    fprintf(stderr, "Data transferred:\t%12.2f MB\n",        data_get_megabytes(data)); /*%12llu*/
+    fprintf(stderr, "Response time:\t\t%12.2f secs\n",       data_get_response_time(data));
+    fprintf(stderr, "Transaction rate:\t%12.2f trans/sec\n", data_get_transaction_rate(data));
+    fprintf(stderr, "Throughput:\t\t%12.2f MB/sec\n",        data_get_throughput(data));
+    fprintf(stderr, "Concurrency:\t\t%12.2f\n",              data_get_concurrency(data));
+    fprintf(stderr, "Successful transactions:%12u\n",        data_get_code(data)); 
+    if (my.debug) {
+      fprintf(stderr, "HTTP OK received:\t%12u\n",             data_get_okay(data));
+    }
+    fprintf(stderr, "Failed transactions:\t%12u\n",          my.failed);
+    fprintf(stderr, "Longest transaction:\t%12.2f\n",        data_get_highest(data));
+    fprintf(stderr, "Shortest transaction:\t%12.2f\n",       data_get_lowest(data));
+    fprintf(stderr, " \n");
   }
-  array_destroy(my.lurl);
-  xfree(client);
-
-  if (my.get) {
-    if (data_get_ok200(D) > 0) {
-       exit(EXIT_SUCCESS);
-    } else {
-      if (!my.quiet) echo("[done]\n");
-      exit(EXIT_FAILURE);
+  if (my.mark)    mark_log_file(my.markstr);
+  if (my.logging) {
+    log_transaction(data);
+    if (my.failures > 0 && my.failed >= my.failures) {
+      mark_log_file("siege aborted due to excessive socket failure.");
     }
   }
 
   /**
-   * take a short nap  for  cosmetic  effect
-   * this does NOT affect performance stats.
+   * Let's clean up after ourselves....
    */
-  pthread_usleep_np(10000);
-  if (my.verbose)
-    fprintf(stderr, "done.\n");
-  else
-    fprintf(stderr, "\b      done.\n");
+  data       = data_destroy(data);
+  urls       = array_destroyer(urls, (void*)url_destroy);
+  browsers   = array_destroyer(browsers, (void*)browser_destroy);
+  cookies    = hash_destroy(cookies);
+  my.cookies = cookies_destroy(my.cookies);
 
-  /**
-   * prepare and print statistics.
-   */
-  if (my.failures > 0 && my.failed >= my.failures) {
-    fprintf(stderr, "%s aborted due to excessive socket failure; you\n", program_name);
-    fprintf(stderr, "can change the failure threshold in $HOME/.%src\n", program_name);
-  }
-  fprintf(stderr, "\nTransactions:\t\t%12u hits\n",        data_get_count(D));
-  fprintf(stderr, "Availability:\t\t%12.2f %%\n",          data_get_count(D)==0 ? 0 :
-                                                           (double)data_get_count(D) /
-                                                           (data_get_count(D)+my.failed)
-                                                           *100
-  );
-  fprintf(stderr, "Elapsed time:\t\t%12.2f secs\n",        data_get_elapsed(D));
-  fprintf(stderr, "Data transferred:\t%12.2f MB\n",        data_get_megabytes(D)); /*%12llu*/
-  fprintf(stderr, "Response time:\t\t%12.2f secs\n",       data_get_response_time(D));
-  fprintf(stderr, "Transaction rate:\t%12.2f trans/sec\n", data_get_transaction_rate(D));
-  fprintf(stderr, "Throughput:\t\t%12.2f MB/sec\n",        data_get_throughput(D));
-  fprintf(stderr, "Concurrency:\t\t%12.2f\n",              data_get_concurrency(D));
-  fprintf(stderr, "Successful transactions:%12u\n",        data_get_code(D)); 
-  if (my.debug) {
-    fprintf(stderr, "HTTP OK received:\t%12u\n",             data_get_ok200(D));
-  }
-  fprintf(stderr, "Failed transactions:\t%12u\n",          my.failed);
-  fprintf(stderr, "Longest transaction:\t%12.2f\n",        data_get_highest(D));
-  fprintf(stderr, "Shortest transaction:\t%12.2f\n",       data_get_lowest(D));
-  fprintf(stderr, " \n");
-  if(my.mark)    mark_log_file(my.markstr);
-  if(my.logging) log_transaction(D);
-
-  data_destroy(D);
   if (my.url == NULL) {
-    for (x = 0; x < my.length; x++)
-      xfree(lines->line[x]);
+    for (i = 0; i < my.length; i++)
+      xfree(lines->line[i]);
     xfree(lines->line);
     xfree(lines);
   } else {
@@ -589,15 +552,5 @@ main(int argc, char *argv[])
     xfree(lines);
   }
 
-  pthread_mutex_destroy( &(cookie->mutex));
-
-  /** 
-   * I should probably take a deeper look 
-   * at cookie content to free it but at 
-   * this point we're two lines from exit
-   */
-  xfree (cookie);
-  xfree (my.url);
-
   exit(EXIT_SUCCESS);  
 } /* end of int main **/
