@@ -87,11 +87,6 @@ char *strchr (), *strrchr ();
 # include <errno.h>
 #endif /* HAVE_ERRNO_H */
 
-#ifndef HAVE_SNPRINTF
-# define portable_snprintf  snprintf
-# define portable_vsnprintf vsnprintf
-#endif
-
 #ifndef  PTHREAD_CREATE_JOINABLE
 # define PTHREAD_CREATE_JOINABLE  0
 #endif /*PTHREAD_CREATE_JOINABLE*/ 
@@ -116,9 +111,8 @@ int strlen();
 #include <url.h>
 #include <auth.h>
 #include <array.h>
-#include <joedog/joedog.h>
+#include <cookies.h>
 #include <joedog/boolean.h>
-#include <joedog/joepath.h>
 
 #define  MXCHLD         1024
 #define  MSG_NOERROR    010000
@@ -140,6 +134,11 @@ int strlen();
 # define INT_MAX (~0 - INT_MIN)
 #endif
 
+typedef struct {
+  int  index;
+  char **line;
+} LINES;
+
 void display_help();
 void display_version(BOOLEAN b);
 
@@ -153,16 +152,19 @@ struct CONFIG
 {
   BOOLEAN logging;      /* boolean, log transactions to log file   */
   BOOLEAN shlog;        /* show log file configuration directive.  */
+  int     limit;        /* Limits the thread count to int          */
   char    *url;         /* URL for the single hit invocation.      */
   char    logfile[128]; /* alternative user defined simbot.log     */ 
   BOOLEAN verbose;      /* boolean, verbose output to screen       */
   BOOLEAN quiet;        /* boolean, turn off all output to screen  */
+  BOOLEAN parser;       /* boolean, turn on/off the HTML parser    */
   BOOLEAN csv;          /* boolean, display verbose output in CSV  */
   BOOLEAN fullurl;      /* boolean, display full url in verbose    */
   BOOLEAN display;      /* boolean, display the thread id verbose  */
   BOOLEAN config;       /* boolean, prints the configuration       */
+  BOOLEAN color;        /* boolean, true for color, false for not  */
   int     cusers;       /* default concurrent users value.         */
-  int     delay;        /* range for random time delay, see -d     */
+  float   delay;        /* range for random time delay, see -d     */
   int     timeout;      /* socket connection timeout value, def:10 */
   BOOLEAN bench;        /* signifies a benchmarking run, no delay  */
   BOOLEAN internet;     /* use random URL selection if TRUE        */
@@ -172,6 +174,7 @@ struct CONFIG
   int     reps;         /* reps to run the test, default infinite  */ 
   char    file[128];    /* urls.txt file, default in joepath.h     */
   int     length;       /* length of the urls array, made global   */
+  LINES * nomap;        /* list of hosts to not follow             */
   BOOLEAN debug;        /* boolean, undocumented debug command     */
   BOOLEAN chunked;      /* boolean, accept chunked encoding        */
   BOOLEAN unique;       /* create unique files for upload          */
@@ -179,12 +182,10 @@ struct CONFIG
   BOOLEAN mark;         /* signifies a log file mark req.          */ 
   char    *markstr;     /* user defined string value to mark file  */
   int     protocol;     /* 0=HTTP/1.0; 1=HTTP/1.1                  */
-  BOOLEAN cookies;      /* to use cookies or not to use cookies    */
+  COOKIES cookies;      /* cookies    */
   char uagent[256];     /* user defined User-Agent string.         */
   char encoding[256];   /* user defined Accept-Encoding string.    */
   char conttype[256];   /* user defined default content type.      */
-  char *username;       /* DEPRECATED!! */
-  char *password;       /* DEPRECATED!! */ 
   int  bids;            /* W & P authorization bids before failure */
   AUTH auth;
   BOOLEAN keepalive;    /* boolean, connection keep-alive value    */
@@ -220,16 +221,6 @@ struct CONFIG
 };
 
 
-/**
- * struct LINES, read by read_cfg_file
- * in config.c 
- */
-typedef struct
-{
-  int  index;
-  char **line;
-} LINES;
-
 #if INTERN
 # define EXTERN /* */
 #else
