@@ -1,7 +1,7 @@
 /**
  * SSL Thread Safe Setup Functions.
  *
- * Copyright (C) 2002-2014 by
+ * Copyright (C) 2002-2016 by
  * Jeffrey Fulmer - <jeff@joedog.org>, et al. 
  * This file is distributed as part of Siege
  *
@@ -44,7 +44,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <stralloc.h>
+#include <memory.h>
 #include <pthread.h>
+#include <notify.h>
 #include <errno.h>
 #include <joedog/defs.h>
 
@@ -63,7 +66,7 @@ private  void SSL_pthreads_locking_callback(int m, int t, char *f, int l);
 #endif/*HAVE_SSL*/
 
 BOOLEAN
-SSL_initialize(CONN *C)
+SSL_initialize(CONN *C, const char *servername)
 {
 #ifdef HAVE_SSL
   int  i;
@@ -90,7 +93,7 @@ SSL_initialize(CONN *C)
     my.ssl_ciphers = stralloc(SSL_DEFAULT_CIPHER_LIST);
   } 
 
-  C->method = SSLv23_client_method();
+  C->method = (SSL_METHOD *)SSLv23_client_method();
   if(C->method==NULL){
     SSL_error_stack();
     return FALSE;
@@ -102,6 +105,7 @@ SSL_initialize(CONN *C)
   } 
 
   SSL_CTX_set_mode(C->ctx, SSL_MODE_ENABLE_PARTIAL_WRITE|SSL_MODE_ACCEPT_MOVING_WRITE_BUFFER);
+  SSL_CTX_set_options(C->ctx, SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3);
   SSL_CTX_set_session_cache_mode(C->ctx, SSL_SESS_CACHE_BOTH);
   SSL_CTX_set_timeout(C->ctx, my.ssl_timeout);
   if(my.ssl_ciphers){
@@ -111,37 +115,50 @@ SSL_initialize(CONN *C)
     }
   }
 
-  if(my.ssl_cert){
-    if(!SSL_CTX_use_certificate_chain_file(C->ctx, my.ssl_cert)){
+  if (my.ssl_cert) {
+    if (!SSL_CTX_use_certificate_chain_file(C->ctx, my.ssl_cert)) {
       SSL_error_stack(); /* dump the error stack */
       NOTIFY(ERROR, "Error reading certificate file: %s", my.ssl_cert);
     }
-    for(i=0; i<3; i++){
-      if(SSL_CTX_use_PrivateKey_file(C->ctx, my.ssl_key, SSL_FILETYPE_PEM))
+    for (i=0; i<3; i++) {
+      if (SSL_CTX_use_PrivateKey_file(C->ctx, my.ssl_key, SSL_FILETYPE_PEM))
         break;
-      if(i<2 && ERR_GET_REASON(ERR_peek_error())==EVP_R_BAD_DECRYPT){
+      if (i<2 && ERR_GET_REASON(ERR_peek_error())==EVP_R_BAD_DECRYPT) {
         SSL_error_stack(); /* dump the error stack */
         NOTIFY(WARNING, "Wrong pass phrase: retrying");
         continue;
       }
     }
 
-    if(!SSL_CTX_check_private_key(C->ctx)){
+    if (!SSL_CTX_check_private_key(C->ctx)) {
       NOTIFY(ERROR, "Private key does not match the certificate");
       return FALSE;
     }
   }  
 
   C->ssl = SSL_new(C->ctx);
-  if(C->ssl==NULL){
+#if defined(SSL_CTRL_SET_TLSEXT_HOSTNAME)
+  SSL_ctrl(C->ssl, SSL_CTRL_SET_TLSEXT_HOSTNAME, TLSEXT_NAMETYPE_host_name, (char *)servername);
+#endif/*SSL_CTRL_SET_TLSEXT_HOSTNAME*/
+
+  if (C->ssl==NULL) {
     SSL_error_stack();
     return FALSE;
   }
+
   SSL_set_fd(C->ssl, C->sock);
   serr = SSL_connect(C->ssl);
+  if (serr != 1) {
+    SSL_error_stack();
+    NOTIFY(ERROR, "Failed to make an SSL connection: %d", SSL_get_error(C->ssl, serr));
+    return FALSE;
+  }
   return TRUE;
 #else
   C->nossl = TRUE;
+  NOTIFY(
+    ERROR, "HTTPS requires libssl: Unable to reach %s with this protocol", servername
+  ); // this message is mainly intended to silence the compiler
   return FALSE;
 #endif/*HAVE_SSL*/
 }
@@ -189,19 +206,32 @@ void
 SSL_thread_cleanup(void) 
 {
   int x;
-  
+
+  xfree(my.ssl_ciphers);
+ 
   CRYPTO_set_locking_callback(NULL);
-  for(x = 0; x < CRYPTO_num_locks(); x++){
+  for (x = 0; x < CRYPTO_num_locks(); x++) {
     pthread_mutex_destroy(&(lock_cs[x]));
   }
-  if(lock_cs!=(pthread_mutex_t *)NULL) { 
+  if (lock_cs!=(pthread_mutex_t *)NULL) { 
     OPENSSL_free(lock_cs); 
     lock_cs=(pthread_mutex_t *)NULL; 
   }
-  if(lock_count!=(long *)NULL){ 
+  if (lock_count!=(long *)NULL) {  
     OPENSSL_free(lock_count); 
     lock_count=(long *)NULL; 
   }
+  sk_SSL_COMP_free(SSL_COMP_get_compression_methods());
+  CRYPTO_cleanup_all_ex_data();
+  ERR_remove_state(0);
+  ERR_free_strings();
+  EVP_cleanup();
+  CRYPTO_cleanup_all_ex_data();
+#if (OPENSSL_VERSION_NUMBER < 0x10100000L) || defined(OPENSSL_USE_DEPRECATED)
+   ERR_remove_state(0);
+#else
+   ERR_remove_thread_state(NULL);
+#endif
 }
 
 void 
